/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Struts", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.struts.webapp.example.oracle;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import javax.sql.DataSource;
import org.apache.struts.action.Action;
import org.apache.struts.action.Tcr;
import org.apache.struts.action.TcrAction;
import org.apache.struts.action.TcrForm;

/**
 * Fill out the list of database tables for the selected database schema
 * from the <code>java.sql.DatabaseMetaData</code> object.
 *
 * @author Alec Missine
 */
public class SelectSchema extends TcrAction {

	protected Tcr newTcr( TcrForm form )
	{
		return new TimeConsumingRequest( form );
	}

	static class TimeConsumingRequest extends Tcr {

		String schema;

		TimeConsumingRequest( TcrForm form ) { 
			super( form );
			schema = ((SchemaForm)form).getSchema();
		}

		ArrayList tables = new ArrayList();

		public void run() 
		{
//long t0 = System.currentTimeMillis(), t1 = 0;
			String types[] = {"TABLE", "VIEW"};
			try { 
//Thread.sleep( 5000 );
				Connection connection = ((DataSource)form.getServlet().
					getServletContext().getAttribute( Action.DATA_SOURCE_KEY )).
					getConnection();
				DatabaseMetaData dbmd = connection.getMetaData();
				ResultSet rs = dbmd.getTables( null, schema, "%", types );
				while( rs.next() ) {
					tables.add( rs.getString( 3 ) );
				}
				done();
//System.out.println( getClass().getName() + ".run: completed in " + ((t1 = System.currentTimeMillis()) - t0) + " ms" );
				rs.close();
				connection.close();
			} catch( Exception e ) { e.printStackTrace(); }
//System.out.println( getClass().getName() + ".run: closed in " + (System.currentTimeMillis() - t1) + " ms" );
		}

		protected void copyTo( TcrForm form ) 
		{
			((SchemaForm)form).setTables( tables );
			((SchemaForm)form).setSchema( schema );
		}
	}
}
